/* Copyright (C) 2009, 2010, 2011, 2012 Keith Crane

This file is part DFILE Tools.

DFILE Tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

DFILE Tools is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with DFILE Tools; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <limits.h>
#include "tbox.h"
#include "dfile.h"
#include "sexpr.h"
#include "dfile_utility.h"
#include "dfile_join.h"


/*
** This function assigns information from control structure.
*/
int assign_join( join_ctl_t **join_ctl_tbl, unsigned short *join_ctl_tbl_cnt, sexpr_t *dfile, unsigned short tuple_cnt, dfile_tag_t *dfile_tag_tbl, unsigned short dfile_tag_tbl_cnt )
{
	char	*section;
	int	ret;
	sexpr_t	*dfile_attribute;
	join_ctl_t	*new;
	size_t	alloc_size;

	assert( join_ctl_tbl != (join_ctl_t **)0 );
	assert( join_ctl_tbl_cnt != (unsigned short *)0 );

	DEBUG_FUNC_START;

	if ( dfile == (sexpr_t *)0 || SEXPR_CAR_TYPE( dfile ) != list_sexpr ) {
		control_syntax_error( "expected a list", __FILE__, __LINE__, tuple_cnt );
		RETURN_INT( -1 );
	}

	*join_ctl_tbl = (join_ctl_t *)0;
	*join_ctl_tbl_cnt = (unsigned short)0;
	ret = 0;

	while ( dfile != (sexpr_t *)0 ) {

		alloc_size = ( (size_t)*join_ctl_tbl_cnt + (size_t)1 ) * sizeof( join_ctl_t );
		new = (join_ctl_t *)realloc( *join_ctl_tbl, alloc_size );

		if ( new == (join_ctl_t *)0 ) {
			UNIX_ERROR( "realloc() failed" );
			RETURN_INT( -1 );
		}

		*join_ctl_tbl = new;
		new += *join_ctl_tbl_cnt;
		++*join_ctl_tbl_cnt;

		(void) memset( (void *)new, 0, sizeof( join_ctl_t ) );

		new->blocks_per_buffer_cnt = (unsigned short)2;
		new->buffer_cnt = (unsigned short)1;

		/*
		** Set initial read flag to true.
		*/
		new->initial_read_flag = 1;

		dfile_attribute = SEXPR_CAR_LIST( dfile );

		if ( SEXPR_CAR_TYPE( dfile_attribute ) != string_sexpr ) {
			control_syntax_error( "expected a string", __FILE__, __LINE__, tuple_cnt );
			RETURN_INT( -1 );
		}

		section = SEXPR_CAR_STRING( dfile_attribute );

		if ( Debug ) {
			(void) fprintf( stderr, "section [%s]\n", section );
		}

		assert( SEXPR_CDR_TYPE( dfile_attribute ) == list_sexpr );

		dfile_attribute = SEXPR_CDR_LIST( dfile_attribute );

		if ( dfile_attribute == (sexpr_t *)0 || SEXPR_CAR_TYPE( dfile_attribute ) != string_sexpr ) {
			control_syntax_error( "expected a string", __FILE__, __LINE__, tuple_cnt );
			RETURN_INT( -1 );
		}

		if ( Debug ) {
			(void) fprintf( stderr, "attribute [%s]\n", SEXPR_CAR_STRING( dfile_attribute ) );
		}

		if ( strcasecmp( section, "dfile" ) != 0 ) {
			control_syntax_error( "expected dfile literal", __FILE__, __LINE__, tuple_cnt );
			RETURN_INT( -1 );
		}

		new->dfile_name = SEXPR_CAR_STRING( dfile_attribute );

		if ( Debug ) {
			fprintf( stderr, "found dfile [%s]\n", new->dfile_name );
		}

		assert( SEXPR_CDR_TYPE( dfile_attribute ) == list_sexpr );

		ret = assign_join_dfile( new, SEXPR_CDR_LIST( dfile_attribute ), tuple_cnt );

		if ( ret == -1 ) {
			break;
		}

		ret = append_tag( &new->tag, &new->tag_cnt, dfile_tag_tbl, dfile_tag_tbl_cnt );

		if ( ret == -1 ) {
			break;
		}

		dfile = SEXPR_CDR_LIST( dfile );

	}

	RETURN_INT( ret );
}
