/* Copyright (C) 2009, 2010, 2011, 2012 Keith Crane

This file is part DFILE Tools.

DFILE Tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

DFILE Tools is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with DFILE Tools; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "tbox.h"
#include "dfile.h"
#include "sexpr.h"
#include "dfile_utility.h"
#include "dfile_join.h"


/*
** This is a function is used to assist debugging. It prints contents of the
** output_ctl_t structure.
*/
void dump_output_ctl( output_ctl_t *output )
{
	unsigned short	ndx;
	dfile_tag_t	*dfile_tag;

	assert( output != (output_ctl_t *)0 );

	DEBUG_FUNC_START;

	(void) fputs( "output:\n", stderr );

	(void) fprintf( stderr, "\tdfile_name = [%s]\n", output->dfile_name );

	(void) fprintf( stderr, "\trecord_filter = %p\n", output->record_filter );

	(void) fputs( "\tdfile tag: <tag> <tag value>\n", stderr );
	dfile_tag = output->tag;
	for ( ndx = (unsigned short)0; ndx < output->tag_cnt; ++ndx ) {
		(void) fprintf( stderr, "\t\t%d. [%s] [%s]\n", ndx + 1, dfile_tag->tag, dfile_tag->tag_value );
		++dfile_tag;
	}

	(void) fprintf( stderr, "\tblocks_per_buffer_cnt = %hu\n", output->blocks_per_buffer_cnt );
	(void) fprintf( stderr, "\tbuffer_cnt = %hu\n", output->buffer_cnt );

	switch ( output->dfile_open_mode ) {
	case Dfile_trunc:
		(void) fputs( "\tdfile open mode is Dfile_trunc\n", stderr );
		break;
	case Dfile_append:
		(void) fputs( "\tdfile open mode is Dfile_append\n", stderr );
		break;
	default:
		(void) fprintf( stderr, "\tdfile open mode is unknown (%d)\n", output->dfile_open_mode );
	}

	(void) fputc( '\n', stderr );

	RETURN_VOID;
}
